@@ -36,19 +36,26 @@ struct _GnomeDbBrowserPrivate {
 };
 
 static void gnome_db_browser_class_init (GnomeDbBrowserClass *klass);
-static void gnome_db_browser_init (GnomeDbBrowser *browser);
-static void gnome_db_browser_destroy (GnomeDbBrowser *browser, gpointer user_data);
+static void gnome_db_browser_init       (GnomeDbBrowser *browser);
+static void gnome_db_browser_destroy    (GnomeDbBrowser *browser, gpointer user_data);
 
 static GtkWidget *create_table_detail (GnomeDbBrowser *browser, const gchar *name);
 
-static void refresh_list_cb (GtkWidget *w, gpointer user_data);
-static void show_extra_info_cb (GtkWidget *w, gpointer user_data);
-static void show_table_data_cb (GtkWidget *w, gpointer user_data);
+static void refresh_list_cb         (GtkWidget *w, gpointer user_data);
+static void show_extra_info_cb      (GtkWidget *w, gpointer user_data);
+static void show_table_data_cb      (GtkWidget *w, gpointer user_data);
 static void switch_notebook_page_cb (GtkNotebook *notebook,
 				     GtkNotebookPage *page,
 				     guint page_num,
 				     gpointer user_data);
 
+/* the arguments it takes */
+enum {
+	ARG_0,
+	ARG_GDA_CONNECTION,
+};
+
+/* signals for this class */
 enum {
 	CONNECTION_CHANGED,
 	LAST_SIGNAL
@@ -58,7 +65,7 @@ static gint db_browser_signals[LAST_SIGNAL] = { 0, };
 static GnomeUIInfo tables_toolbar[] = {
 	{ GNOME_APP_UI_ITEM, N_("Data"), N_("Show table data"),
 	  show_table_data_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-	  GNOME_STOCK_MENU_NEW, 0, 0, NULL },
+	  GNOME_STOCK_MENU_SEARCH, 0, 0, NULL },
 	GNOMEUIINFO_END
 };
 static struct {
@@ -76,7 +83,11 @@ static struct {
 	{ N_("Procedures"), GDA_Connection_GDCN_SCHEMA_PROCS, GNOME_STOCK_MENU_EXEC,
 	  GNOME_STOCK_PIXMAP_EXEC, NULL, NULL },
 	{ N_("Types"), GDA_Connection_GDCN_SCHEMA_PROV_TYPES, GNOME_STOCK_MENU_FONT,
-	  GNOME_STOCK_PIXMAP_FONT, NULL, NULL }
+	  GNOME_STOCK_PIXMAP_FONT, NULL, NULL },
+	{ N_("Aggregates"), GDA_Connection_GDCN_SCHEMA_AGGREGATES, GNOME_STOCK_MENU_BACK,
+	  GNOME_STOCK_PIXMAP_BACK, NULL, NULL },
+	{ N_("Sequences"), GDA_Connection_GDCN_SCHEMA_SEQUENCES, GNOME_STOCK_MENU_SCORES,
+	  GNOME_STOCK_PIXMAP_SCORES, NULL, NULL }
 };
 static GnomeUIInfo actions_toolbar[] = {
 	{ GNOME_APP_UI_ITEM, N_("New"), N_("Create new object of selected type"),
@@ -101,6 +112,53 @@ static GtkVBoxClass *parent_class = NULL;
 /*
  * Private functions
  */
+static void
+create_detail (GnomeDbBrowser *browser, gint current_page)
+{
+	gchar *object_name;
+	GnomeDbList *list;
+
+	g_return_if_fail (GNOME_DB_IS_BROWSER (browser));
+
+	list = (GnomeDbList *) gtk_notebook_get_nth_page (browser->priv->object_selector,
+							  current_page);
+	if (!GNOME_DB_IS_LIST (list))
+		return;
+
+	object_name = gnome_db_list_get_string (list);
+
+	/* destroy previous widgets */
+	if (browser->priv->detail != NULL) {
+		gtk_widget_destroy(browser->priv->detail);
+		browser->priv->detail = NULL;
+	}
+	if (GTK_IS_WIDGET(browser->priv->detail_toolbar)) {
+		gtk_widget_destroy(browser->priv->detail_toolbar);
+		browser->priv->detail_toolbar = NULL;
+	}
+
+	/* create detail for this object */
+	if (object_managers[current_page].create_detail && object_name) {
+		browser->priv->detail =
+			object_managers[current_page].create_detail(browser,
+								    object_name);
+		gtk_widget_show(browser->priv->detail);
+		gtk_box_pack_start (GTK_BOX (browser->priv->detail_container),
+				    browser->priv->detail, 1, 1, 0);
+	}
+
+	if (object_managers[current_page].toolbar) {
+		browser->priv->detail_toolbar =
+			gnome_db_new_toolbar_widget(GTK_ORIENTATION_HORIZONTAL,
+						    GTK_TOOLBAR_ICONS,
+						    tables_toolbar,
+						    (gpointer) browser);
+		gtk_table_attach(GTK_TABLE(browser->priv->table),
+				 browser->priv->detail_toolbar,
+				 1, 2, 0, 1, 0, 0, 3, 3);
+	}
+}
+
 static GtkWidget *
 create_table_detail (GnomeDbBrowser *browser, const gchar *name)
 {
@@ -120,6 +178,7 @@ create_table_detail (GnomeDbBrowser *browser, const gchar *name)
 						     name,
 						     GDA_Connection_no_CONSTRAINT);
 		gnome_db_grid_set_recordset (GNOME_DB_GRID (grid), recset);
+		gda_recordset_free (recset);
 	}
 
 	return grid;
@@ -128,17 +187,21 @@ create_table_detail (GnomeDbBrowser *browser, const gchar *name)
 static void
 fill_object_list (GnomeDbBrowser *browser, GnomeDbList *list, GDA_Connection_QType qtype)
 {
-	g_return_if_fail(GNOME_DB_IS_BROWSER(browser));
-	g_return_if_fail(GNOME_DB_IS_LIST(list));
-	g_return_if_fail(IS_GDA_CONNECTION(browser->priv->cnc));
+	g_return_if_fail (GNOME_DB_IS_BROWSER (browser));
+	g_return_if_fail (GNOME_DB_IS_LIST (list));
+	g_return_if_fail (GDA_IS_CONNECTION (browser->priv->cnc));
 
-	if (gda_connection_is_open(browser->priv->cnc)) {
+	if (gda_connection_is_open (browser->priv->cnc)) {
 		GdaRecordset *recset;
 
-		recset = gda_connection_open_schema(browser->priv->cnc, qtype, GDA_Connection_no_CONSTRAINT);
-		gnome_db_list_set_recordset(list, recset, 0);
+		recset = gda_connection_open_schema (browser->priv->cnc,
+						     qtype,
+						     GDA_Connection_no_CONSTRAINT);
+		gnome_db_list_set_recordset (list, recset, 0);
+		gda_recordset_free (recset);
 	}
-	else gnome_db_list_set_recordset(list, NULL, 0);
+	else
+		gnome_db_list_set_recordset (list, NULL, 0);
 }
 
 /**
@@ -149,15 +212,15 @@ connection_closed_cb (GdaConnection *cnc, gpointer user_data)
 {
 	GnomeDbBrowser *browser = (GnomeDbBrowser *) user_data;
 
-	g_return_if_fail(IS_GDA_CONNECTION(cnc));
-	g_return_if_fail(GNOME_DB_IS_BROWSER(browser));
-	g_return_if_fail(browser->priv->cnc == cnc);
+	g_return_if_fail (GDA_IS_CONNECTION (cnc));
+	g_return_if_fail (GNOME_DB_IS_BROWSER (browser));
+	g_return_if_fail (browser->priv->cnc == cnc);
 
 	/* FIXME: clean up all widgets */
 	browser->priv->cnc = NULL;
-	gnome_db_browser_refresh(browser);
+	gnome_db_browser_refresh (browser);
 
-	gtk_signal_emit(GTK_OBJECT(browser), db_browser_signals[CONNECTION_CHANGED]);
+	gtk_signal_emit (GTK_OBJECT (browser), db_browser_signals[CONNECTION_CHANGED]);
 }
 
 static void
@@ -169,35 +232,8 @@ object_selected_cb (GnomeDbList *list, gpointer user_data)
 	g_return_if_fail(GNOME_DB_IS_BROWSER(browser));
 
 	current_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(browser->priv->object_selector));
-	if (current_page >= 0 && current_page < NUMBER_OF_OBJECTS) {
-		/* destroy previous widgets */
-		if (browser->priv->detail != NULL) {
-			gtk_widget_destroy(browser->priv->detail);
-			browser->priv->detail = NULL;
-		}
-		if (GTK_IS_WIDGET(browser->priv->detail_toolbar)) {
-			gtk_widget_destroy(browser->priv->detail_toolbar);
-			browser->priv->detail_toolbar = NULL;
-		}
-
-		/* create detail for this object */
-		if (object_managers[current_page].create_detail) {
-			browser->priv->detail =
-				object_managers[current_page].create_detail(browser,
-															gnome_db_list_get_string(list));
-			gtk_widget_show(browser->priv->detail);
-			gtk_container_add(GTK_CONTAINER(browser->priv->detail_container), browser->priv->detail);
-		}
-
-		if (object_managers[current_page].toolbar) {
-			browser->priv->detail_toolbar = gnome_db_new_toolbar_widget(GTK_ORIENTATION_HORIZONTAL,
-																		GTK_TOOLBAR_ICONS,
-																		tables_toolbar,
-																		(gpointer) browser);
-			gtk_table_attach(GTK_TABLE(browser->priv->table), browser->priv->detail_toolbar,
-							 1, 2, 0, 1, 0, 0, 3, 3);
-		}
-	}
+	if (current_page >= 0 && current_page < NUMBER_OF_OBJECTS)
+		create_detail (browser, current_page);
 }
 
 static void
@@ -226,7 +262,7 @@ refresh_list_cb (GtkWidget *w, gpointer user_data)
 
 	page_num = gtk_notebook_get_current_page(GTK_NOTEBOOK(browser->priv->object_selector));
 	list = (GnomeDbList *) gtk_notebook_get_nth_page(GTK_NOTEBOOK(browser->priv->object_selector),
-													 page_num);
+							 page_num);
 	if (GNOME_DB_IS_LIST(list)) {
 		fill_object_list(browser, list, object_managers[page_num].qtype);
 	}
@@ -243,7 +279,7 @@ show_extra_info_cb (GtkWidget *w, gpointer user_data)
 
 	page_num = gtk_notebook_get_current_page(GTK_NOTEBOOK(browser->priv->object_selector));
 	list = (GnomeDbList *) gtk_notebook_get_nth_page(GTK_NOTEBOOK(browser->priv->object_selector),
-													 page_num);
+							 page_num);
 	if (GNOME_DB_IS_LIST(list)) {
 		gchar *object_name;
 		GdaRecordset *recset;
@@ -306,7 +342,7 @@ show_table_data_cb (GtkWidget *w, gpointer user_data)
 	page_num = gtk_notebook_get_current_page(GTK_NOTEBOOK(browser->priv->object_selector));
 	list = (GnomeDbList *) gtk_notebook_get_nth_page(GTK_NOTEBOOK(browser->priv->object_selector),
 							 page_num);
-	if (GNOME_DB_IS_LIST(list) && IS_GDA_CONNECTION(browser->priv->cnc)) {
+	if (GNOME_DB_IS_LIST(list) && GDA_IS_CONNECTION(browser->priv->cnc)) {
 		gchar *object_name;
 
 		object_name = gnome_db_list_get_string(list);
@@ -357,28 +393,55 @@ switch_notebook_page_cb (GtkNotebook *notebook,
 	g_return_if_fail(GNOME_DB_IS_BROWSER(browser));
 	g_return_if_fail(browser->priv != NULL);
 	g_return_if_fail(browser->priv->cnc != NULL);
-	g_return_if_fail(IS_GDA_CONNECTION(browser->priv->cnc));
+	g_return_if_fail(GDA_IS_CONNECTION(browser->priv->cnc));
 
 	list = (GnomeDbList *) gtk_notebook_get_nth_page(notebook, page_num);
-	if (GNOME_DB_IS_LIST(list)) {
-		if (!gnome_db_list_get_recordset(list))
-			fill_object_list(browser, list, object_managers[page_num].qtype);
+	if (!GNOME_DB_IS_LIST (list))
+		return;
 
-		/* destroy previous widgets */
-		if (browser->priv->detail != NULL) {
-			gtk_widget_destroy(browser->priv->detail);
-			browser->priv->detail = NULL;
-		}
-		if (GTK_IS_WIDGET(browser->priv->detail_toolbar)) {
-			gtk_widget_destroy(browser->priv->detail_toolbar);
-			browser->priv->detail_toolbar = NULL;
-		}
-	}
+	if (!gnome_db_list_get_recordset (list))
+		fill_object_list (browser, list, object_managers[page_num].qtype);
+
+	/* create the detail pane for the currently selected object */
+	create_detail (browser, page_num);
 }
 
 /*
  * GnomeDbBrowser interface
  */
+static void
+gnome_db_browser_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+{
+	GnomeDbBrowser *browser = GNOME_DB_BROWSER (object);
+
+	g_return_if_fail (GNOME_DB_IS_BROWSER (browser));
+
+	switch (arg_id) {
+	case ARG_GDA_CONNECTION :
+		GTK_VALUE_OBJECT (*arg) = (GtkObject *) browser->priv->cnc;
+		break;
+	default :
+		arg->type = GTK_TYPE_INVALID;
+		break;
+	}
+}
+
+static void
+gnome_db_browser_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+{
+	GnomeDbBrowser *browser = (GnomeDbBrowser *) object;
+
+	g_return_if_fail (GNOME_DB_IS_BROWSER (browser));
+
+	switch (arg_id) {
+	case ARG_GDA_CONNECTION :
+		gnome_db_browser_set_connection (browser, GTK_VALUE_OBJECT (*arg));
+		break;
+	default :
+		break;
+	}
+}
+
 static void
 gnome_db_browser_class_init (GnomeDbBrowserClass *klass)
 {
@@ -396,6 +459,14 @@ gnome_db_browser_class_init (GnomeDbBrowserClass *klass)
 	gtk_object_class_add_signals(object_class, db_browser_signals, LAST_SIGNAL);
 
 	object_class->destroy = gnome_db_browser_destroy;
+	object_class->get_arg = gnome_db_browser_get_arg;
+	object_class->set_arg = gnome_db_browser_set_arg;
+
+	gtk_object_add_arg_type ("GnomeDbBrowser:GdaConnection",
+				 GNOME_DB_TYPE_BROWSER,
+				 GTK_ARG_READWRITE,
+				 ARG_GDA_CONNECTION);
+
 	klass->connection_changed = NULL;
 }
 
@@ -405,9 +476,9 @@ gnome_db_browser_destroy (GnomeDbBrowser *browser, gpointer data)
 	g_return_if_fail(GNOME_DB_IS_BROWSER(browser));
 
 	gtk_signal_disconnect_by_func(GTK_OBJECT(browser->priv->object_selector),
-								  GTK_SIGNAL_FUNC(switch_notebook_page_cb),
-								  (gpointer) browser);
-	if (IS_GDA_CONNECTION(browser->priv->cnc)) {
+				      GTK_SIGNAL_FUNC(switch_notebook_page_cb),
+				      (gpointer) browser);
+	if (GDA_IS_CONNECTION(browser->priv->cnc)) {
 		gtk_signal_disconnect_by_func(GTK_OBJECT(browser->priv->cnc),
 		                              GTK_SIGNAL_FUNC(connection_closed_cb),
 		                              (gpointer) browser);
@@ -440,17 +511,17 @@ gnome_db_browser_init (GnomeDbBrowser *browser)
 
 		list = gnome_db_list_new(NULL, 0);
 		gtk_signal_connect(GTK_OBJECT(list),
-						   "select_row",
-						   GTK_SIGNAL_FUNC(object_selected_cb),
-						   (gpointer) browser);
+				   "select_row",
+				   GTK_SIGNAL_FUNC(object_selected_cb),
+				   (gpointer) browser);
 		gtk_signal_connect(GTK_OBJECT(list),
-						   "add_row",
-						   GTK_SIGNAL_FUNC(row_added_cb),
-						   (gpointer) object_managers[n].pixmap_file);
+				   "add_row",
+				   GTK_SIGNAL_FUNC(row_added_cb),
+				   (gpointer) object_managers[n].pixmap_file);
 		gtk_widget_show(list);
 		gtk_notebook_append_page(GTK_NOTEBOOK(browser->priv->object_selector),
-								 list,
-								 gtk_label_new(object_managers[n].label));
+					 list,
+					 gtk_label_new(object_managers[n].label));
 	}
 
 	/* create detail container */
@@ -461,7 +532,8 @@ gnome_db_browser_init (GnomeDbBrowser *browser)
 										  (gpointer) browser);
 	gtk_table_attach(GTK_TABLE(browser->priv->table), toolbar, 0, 1, 0, 1, 0, 0, 3, 3);
 
-	browser->priv->detail_container = gnome_db_new_frame_widget(NULL);
+	browser->priv->detail_container = gtk_vbox_new (TRUE, 0);
+	gtk_widget_show (browser->priv->detail_container);
 	gtk_table_attach(GTK_TABLE(browser->priv->table), browser->priv->detail_container, 0, 6, 1, 2,
 					 GTK_FILL | GTK_SHRINK | GTK_EXPAND,
 					 GTK_FILL | GTK_SHRINK | GTK_EXPAND,
@@ -471,9 +543,9 @@ gnome_db_browser_init (GnomeDbBrowser *browser)
 
 	/* connect signals */
 	gtk_signal_connect(GTK_OBJECT(browser->priv->object_selector),
-					   "switch_page",
-					   GTK_SIGNAL_FUNC(switch_notebook_page_cb),
-					   (gpointer) browser);
+			   "switch_page",
+			   GTK_SIGNAL_FUNC(switch_notebook_page_cb),
+			   (gpointer) browser);
 }
 
 GtkType
@@ -498,55 +570,106 @@ gnome_db_browser_get_type (void)
 
 /**
  * gnome_db_browser_new
+ * @cnc: a #GdaConnection
+ *
+ * Creates a new #GnomeDbBrowser widget. You can associate a connection
+ * to the widget at creation time by calling this function with an already
+ * established connection as the @cnc parameter. You can also associate
+ * later on another connection to the #GnomeDbBrowser widget by calling
+ * #gnome_db_browser_set_connection.
+ *
+ * Returns: the newly created browser widget
  */
 GtkWidget *
 gnome_db_browser_new (GdaConnection *cnc)
 {
 	GnomeDbBrowser* browser;
 	
-	browser = GNOME_DB_BROWSER(gtk_type_new(gnome_db_browser_get_type()));
-	if (IS_GDA_CONNECTION(cnc)) {
-		gnome_db_browser_set_connection(browser, cnc);
+	browser = GNOME_DB_BROWSER (gtk_type_new (gnome_db_browser_get_type ()));
+	if (GDA_IS_CONNECTION (cnc)) {
+		gnome_db_browser_set_connection (browser, cnc);
 	}
-	return GTK_WIDGET(browser);
+	return GTK_WIDGET (browser);
 }
 
 /**
  * gnome_db_browser_get_connection
+ * @browser: a #GnomeDbBrowser widget
+ *
+ * Gets the #GdaConnection associated with the given browser widget.
+ *
+ * Returns: the connection being used by the browser widget
  */
 GdaConnection *
 gnome_db_browser_get_connection (GnomeDbBrowser *browser)
 {
-	g_return_val_if_fail(GNOME_DB_IS_BROWSER(browser), NULL);
-	g_return_val_if_fail(browser->priv != NULL, NULL);
+	g_return_val_if_fail (GNOME_DB_IS_BROWSER (browser), NULL);
+	g_return_val_if_fail (browser->priv != NULL, NULL);
 	return browser->priv->cnc;
 }
 
 /**
  * gnome_db_browser_set_connection
+ * @browser: a #GnomeDbBrowser widget
+ * @cnc: a #GdaConnection
+ *
+ * Associates a #GdaConnection object with the given #GnomeDbBrowser widget.
+ * By doing so, the browser widget will react accordingly by refreshing
+ * all the information it is showing, so that it reflects the structure
+ * of the new connection.
  */
 void
 gnome_db_browser_set_connection (GnomeDbBrowser *browser, GdaConnection *cnc)
 {
-	g_return_if_fail(GNOME_DB_IS_BROWSER(browser));
-	g_return_if_fail(browser->priv != NULL);
+	g_return_if_fail (GNOME_DB_IS_BROWSER(browser));
+	g_return_if_fail (browser->priv != NULL);
 
 	browser->priv->cnc = cnc;
-	gnome_db_browser_refresh(browser);
+	gnome_db_browser_refresh (browser);
 
 	/* connect callback to "close" signal */
-	if (IS_GDA_CONNECTION(cnc)) {
-		gtk_signal_connect(GTK_OBJECT(cnc),
-		                   "close",
-		                   GTK_SIGNAL_FUNC(connection_closed_cb),
-		                   (gpointer) browser);
+	if (GDA_IS_CONNECTION (cnc)) {
+		gtk_signal_connect (GTK_OBJECT (cnc),
+				    "close",
+				    GTK_SIGNAL_FUNC (connection_closed_cb),
+				    (gpointer) browser);
 	}
 
-	gtk_signal_emit(GTK_OBJECT(browser), db_browser_signals[CONNECTION_CHANGED]);
+	gtk_signal_emit (GTK_OBJECT (browser), db_browser_signals[CONNECTION_CHANGED]);
+}
+
+/**
+ * gnome_db_browser_show_of_type
+ * @browser: A #GnomeDbBrowser widget
+ * @qtype: The type of the objects to show
+ *
+ * Activates, in the given #GnomeDbBrowser widget, the list of
+ * objects for the given type. For a list of valid types, the GDA schemas
+ * constraint type is used (#GDA_Connection_QType), so any valid constraint
+ * type can be passed to this function.
+ *
+ * Note, however, that the #GnomeDbBrowser does not know how to show all kinds
+ * of objects, so this may not work for some schema types.
+ */
+void
+gnome_db_browser_show_of_type (GnomeDbBrowser *browser, GDA_Connection_QType qtype)
+{
+	gint n;
+
+	g_return_if_fail (GNOME_DB_IS_BROWSER (browser));
+
+	for (n = 0; n < NUMBER_OF_OBJECTS; n++) {
+		if (object_managers[n].qtype == qtype)
+			gtk_notebook_set_page (GTK_NOTEBOOK(browser->priv->object_selector), n);
+	}
 }
 
 /**
  * gnome_db_browser_refresh
+ * @browser: a #GnomeDbBrowser widget
+ *
+ * Refreshes all the information being displayed by the given
+ * #GnomeDbBrowser widget
  */
 void
 gnome_db_browser_refresh (GnomeDbBrowser *browser)
